@@ -24,9 +24,9 @@ parse_model <- function(model, backend, ...) {
 .parse_model_cmdstanr <- function(model, silent = 1, ...) {
   require_package("cmdstanr")
   temp_file <- cmdstanr::write_stan_file(model)
-  if (cmdstanr::cmdstan_version() >= "2.29.0") {
-    .canonicalize_stan_model(temp_file, overwrite_file = TRUE)
-  }
+  # if (cmdstanr::cmdstan_version() >= "2.29.0") {
+  #   .canonicalize_stan_model(temp_file, overwrite_file = TRUE)
+  # }
   out <- eval_silent(
     cmdstanr::cmdstan_model(temp_file, compile = FALSE, ...),
     type = "message", try = TRUE, silent = silent
@@ -71,7 +71,7 @@ compile_model <- function(model, backend, ...) {
     message("Compiling Stan program...")
   }
   if (use_threading(threads)) {
-    if (utils::packageVersion("rstan") >= 2.26) {
+    if (utils::packageVersion("rstan") >= "2.26") {
       threads_per_chain_def <- rstan::rstan_options("threads_per_chain")
       on.exit(rstan::rstan_options(threads_per_chain = threads_per_chain_def))
       rstan::rstan_options(threads_per_chain = threads$threads)
@@ -97,9 +97,9 @@ compile_model <- function(model, backend, ...) {
   require_package("cmdstanr")
   args <- list(...)
   args$stan_file <- cmdstanr::write_stan_file(model)
-  if (cmdstanr::cmdstan_version() >= "2.29.0") {
-    .canonicalize_stan_model(args$stan_file, overwrite_file = TRUE)
-  }
+  # if (cmdstanr::cmdstan_version() >= "2.29.0") {
+  #   .canonicalize_stan_model(args$stan_file, overwrite_file = TRUE)
+  # }
   if (use_threading(threads)) {
     args$cpp_options$stan_threads <- TRUE
   }
@@ -148,7 +148,7 @@ fit_model <- function(model, backend, ...) {
 
   # some input checks and housekeeping
   if (use_threading(threads)) {
-    if (utils::packageVersion("rstan") >= 2.26) {
+    if (utils::packageVersion("rstan") >= "2.26") {
       threads_per_chain_def <- rstan::rstan_options("threads_per_chain")
       on.exit(rstan::rstan_options(threads_per_chain = threads_per_chain_def))
       rstan::rstan_options(threads_per_chain = threads$threads)
@@ -215,7 +215,8 @@ fit_model <- function(model, backend, ...) {
   } else {
     stop2("Algorithm '", algorithm, "' is not supported.")
   }
-  out <- repair_stanfit_names(out)
+  # TODO: add support for pathfinder and laplace
+  out <- repair_stanfit(out)
   out
 }
 
@@ -242,17 +243,9 @@ fit_model <- function(model, backend, ...) {
     stop2("Argument 'future' is not supported by backend 'cmdstanr'.")
   }
   args <- nlist(data = sdata, seed, init)
-  if (use_threading(threads)) {
-    if (algorithm %in% c("sampling", "fixed_param")) {
-      args$threads_per_chain <- threads$threads
-    } else if (algorithm %in% c("fullrank", "meanfield")) {
-      args$threads <- threads$threads
-    }
-  }
   if (use_opencl(opencl)) {
     args$opencl_ids <- opencl$ids
   }
-  # TODO: exclude variables via 'exclude'
   dots <- list(...)
   args[names(dots)] <- dots
   args[names(control)] <- control
@@ -279,27 +272,39 @@ fit_model <- function(model, backend, ...) {
       iter_warmup = warmup,
       chains, thin,
       parallel_chains = cores,
-      show_messages = !silent,
+      show_messages = silent < 2,
+      show_exceptions = silent == 0,
       fixed_param = algorithm == "fixed_param"
     )
+    if (use_threading(threads)) {
+      args$threads_per_chain <- threads$threads
+    }
     out <- do_call(model$sample, args)
   } else if (algorithm %in% c("fullrank", "meanfield")) {
-    # vb does not support parallel execution
     c(args) <- nlist(iter, algorithm)
+    if (use_threading(threads)) {
+      args$threads <- threads$threads
+    }
     out <- do_call(model$variational, args)
+  } else if (algorithm %in% c("pathfinder")) {
+    if (use_threading(threads)) {
+      args$num_threads <- threads$threads
+    }
+    out <- do_call(model$pathfinder, args)
+  } else if (algorithm %in% c("laplace")) {
+    if (use_threading(threads)) {
+      args$threads <- threads$threads
+    }
+    out <- do_call(model$laplace, args)
   } else {
     stop2("Algorithm '", algorithm, "' is not supported.")
   }
-  # a lot of metadata is not stored via rstan::read_stan_csv
-  metadata <- cmdstanr::read_cmdstan_csv(
-    out$output_files(), variables = "", sampler_diagnostics = ""
+
+  out <- read_csv_as_stanfit(
+    out$output_files(), variables = out$metadata()$stan_variables,
+    model = model, exclude = exclude, algorithm = algorithm
   )
-  # transform into stanfit object for consistent output structure
-  out <- rstan::read_stan_csv(out$output_files())
-  out <- repair_stanfit_names(out)
-  # allow updating the model without recompilation
-  attributes(out)$CmdStanModel <- model
-  attributes(out)$metadata <- metadata
+
   if (empty_model) {
     # allow correct updating of an 'empty' model
     out@sim <- list()
@@ -420,7 +425,7 @@ backend_choices <- function() {
 
 # supported Stan algorithms
 algorithm_choices <- function() {
-  c("sampling", "meanfield", "fullrank", "fixed_param")
+  c("sampling", "meanfield", "fullrank", "pathfinder", "laplace", "fixed_param")
 }
 
 # check if the model was fit the the required backend
@@ -595,8 +600,17 @@ validate_silent <- function(silent) {
   silent
 }
 
+# ensure that variable dimensions at the end are correctly written
+# convert names like b.1.1 to b[1,1]
+repair_variable_names <- function(x) {
+  x <- sub("\\.", "[", x)
+  x <- gsub("\\.", ",", x)
+  x[grep("\\[", x)] <- paste0(x[grep("\\[", x)], "]")
+  x
+}
+
 # repair parameter names of stanfit objects
-repair_stanfit_names <- function(x) {
+repair_stanfit <- function(x) {
   stopifnot(is.stanfit(x))
   if (!length(x@sim$fnames_oi)) {
     # nothing to rename
@@ -606,7 +620,7 @@ repair_stanfit_names <- function(x) {
   # this case happens rarely but might happen when sample_prior = "yes"
   x@sim$fnames_oi <- make.unique(as.character(x@sim$fnames_oi), "__")
   for (i in seq_along(x@sim$samples)) {
-    # rstan::read_stan_csv may have renamed dimension suffixes (#1218)
+    # stanfit may have renamed dimension suffixes (#1218)
     if (length(x@sim$samples[[i]]) == length(x@sim$fnames_oi)) {
       names(x@sim$samples[[i]]) <- x@sim$fnames_oi
     }
@@ -619,33 +633,379 @@ file_refit_options <- function() {
   c("never", "always", "on_change")
 }
 
-.canonicalize_stan_model <- function(stan_file, overwrite_file = TRUE) {
-  if (os_is_windows()) {
-    stanc_cmd <- "bin/stanc.exe"
-  } else {
-    stanc_cmd <- "bin/stanc"
-  }
-  stanc_flags <- c(
-    "--auto-format",
-    "--canonicalize=deprecations,braces,parentheses"
-  )
-  if (cmdstanr::cmdstan_version() >= "2.29.0") {
-    require_package("processx")
-    res <- processx::run(
-      command = stanc_cmd,
-      args = c(stan_file, stanc_flags),
-      wd = cmdstanr::cmdstan_path(),
-      echo = FALSE,
-      echo_cmd = FALSE,
-      spinner = FALSE,
-      stderr_callback = function(x, p) {
-        message(x)
-      },
-      error_on_status = TRUE
-    )
-    if (overwrite_file) {
-      cat(res$stdout, file = stan_file, sep = "\n")
+# canonicalize Stan model file in accordance with the current Stan version
+# this function may no longer be needed due to rstan 2.26+ now being on CRAN
+# for more details see https://github.com/paul-buerkner/brms/issues/1544
+# .canonicalize_stan_model <- function(stan_file, overwrite_file = TRUE) {
+#   cmdstan_mod <- cmdstanr::cmdstan_model(stan_file, compile = FALSE)
+#   out <- utils::capture.output(
+#     cmdstan_mod$format(
+#       canonicalize = list("deprecations", "braces", "parentheses"),
+#       overwrite_file = overwrite_file, backup = FALSE
+#     )
+#   )
+#   paste0(out, collapse = "\n")
+# }
+
+#' Read CmdStan CSV files as a brms-formatted stanfit object
+#'
+#' \code{read_csv_as_stanfit} is used internally to read CmdStan CSV files into a
+#' \code{stanfit} object that is consistent with the structure of the fit slot of a
+#' brmsfit object.
+#'
+#' @param files Character vector of CSV files names where draws are stored.
+#' @param variables Character vector of variables to extract from the CSV files.
+#' @param sampler_diagnostics Character vector of sampler diagnostics to extract.
+#' @param model A compiled cmdstanr model object (optional). Provide this argument
+#'  if you want to allow updating the model without recompilation.
+#' @param exclude Character vector of variables to exclude from the stanfit. Only
+#'  used when \code{variables} is also specified.
+#' @param algorithm The algorithm with which the model was fitted.
+#'   See \code{\link{brm}} for details.
+#'
+#' @return A stanfit object consistent with the structure of the \code{fit}
+#'  slot of a brmsfit object.
+#'
+#' @examples
+#' \dontrun{
+#' # fit a model manually via cmdstanr
+#' scode <- stancode(count ~ Trt, data = epilepsy)
+#' sdata <- standata(count ~ Trt, data = epilepsy)
+#' mod <- cmdstanr::cmdstan_model(cmdstanr::write_stan_file(scode))
+#' stanfit <- mod$sample(data = sdata)
+#'
+#' # feed the Stan model back into brms
+#' fit <- brm(count ~ Trt, data = epilepsy, empty = TRUE, backend = 'cmdstanr')
+#' fit$fit <- read_csv_as_stanfit(stanfit$output_files(), model = mod)
+#' fit <- rename_pars(fit)
+#' summary(fit)
+#' }
+#'
+#' @export
+read_csv_as_stanfit <- function(files, variables = NULL, sampler_diagnostics = NULL,
+                                model = NULL, exclude = "", algorithm = "sampling") {
+  require_package("cmdstanr")
+
+  if (!is.null(variables)) {
+    # ensure that only relevant variables are read from CSV
+    variables <- repair_variable_names(variables)
+    variables <- unique(sub("\\[.+", "", variables))
+    variables <- setdiff(variables, exclude)
+    # temp fix for cmdstanr not recognizing the variable names it produces #1473
+    if (algorithm %in% c("meanfield", "fullrank")) {
+      variables <- ifelse(variables == "lp_approx__", "log_g__", variables)
+    } else if (algorithm %in% "pathfinder") {
+      variables <- setdiff(variables, "lp_approx__")
+    } else if (algorithm %in% "laplace") {
+      variables <- setdiff(variables, c("lp__", "lp_approx__"))
     }
   }
-  res$stdout
+
+  csfit <- cmdstanr::read_cmdstan_csv(
+    files = files, variables = variables,
+    sampler_diagnostics = sampler_diagnostics,
+    format = NULL
+  )
+
+  # @model_name
+  model_name = gsub(".csv", "", basename(files[[1]]))
+
+  # @model_pars
+  model_pars <- csfit$metadata$stan_variables
+  if (!is.null(variables)) {
+    model_pars <- intersect(model_pars, variables)
+  }
+  # special variables will be added back later on
+  special_vars <- c("lp__", "lp_approx__")
+  model_pars <- setdiff(model_pars, special_vars)
+
+  # @par_dims
+  par_dims <- vector("list", length(model_pars))
+  names(par_dims) <- model_pars
+  par_dims <- lapply(par_dims, function(x) integer(0))
+  pdims_num <- ulapply(model_pars, function(x)
+    sum(grepl(paste0("^", x, "\\[.*\\]$"), csfit$metadata$model_params))
+  )
+  par_dims[pdims_num != 0] <-
+    csfit$metadata$stan_variable_sizes[model_pars][pdims_num != 0]
+
+  # @mode
+  mode <- 0L
+
+  # @sim
+  rstan_diagn_order <- c("accept_stat__", "treedepth__", "stepsize__",
+                         "divergent__", "n_leapfrog__", "energy__")
+
+  if (!is.null(sampler_diagnostics)) {
+    rstan_diagn_order <- rstan_diagn_order[rstan_diagn_order %in% sampler_diagnostics]
+  }
+
+  res_vars <- c(".chain", ".iteration", ".draw")
+  if ("post_warmup_draws" %in% names(csfit)) {
+    # for MCMC samplers
+    n_chains <- max(
+      nchains(csfit$warmup_draws),
+      nchains(csfit$post_warmup_draws)
+    )
+    n_iter_warmup <- niterations(csfit$warmup_draws)
+    n_iter_sample <- niterations(csfit$post_warmup_draws)
+    if (n_iter_warmup > 0) {
+      csfit$warmup_draws <- as_draws_df(csfit$warmup_draws)
+      csfit$warmup_sampler_diagnostics <-
+        as_draws_df(csfit$warmup_sampler_diagnostics)
+    }
+    if (n_iter_sample > 0) {
+      csfit$post_warmup_draws <- as_draws_df(csfit$post_warmup_draws)
+      csfit$post_warmup_sampler_diagnostics <-
+        as_draws_df(csfit$post_warmup_sampler_diagnostics)
+    }
+
+    # called 'samples' for consistency with rstan
+    samples <- rbind(csfit$warmup_draws, csfit$post_warmup_draws)
+    # manage memory
+    csfit$warmup_draws <- NULL
+    csfit$post_warmup_draws <- NULL
+
+    # prepare sampler diagnostics
+    diagnostics <- rbind(
+      csfit$warmup_sampler_diagnostics,
+      csfit$post_warmup_sampler_diagnostics
+    )
+
+    # manage memory
+    csfit$warmup_sampler_diagnostics <- NULL
+    csfit$post_warmup_sampler_diagnostics <- NULL
+    # convert to regular data.frame
+    diagnostics <- as.data.frame(diagnostics)
+    diag_chain_ids <- diagnostics$.chain
+    diagnostics[res_vars] <- NULL
+
+  } else if ("draws" %in% names(csfit)) {
+    # for variational inference "samplers"
+    n_chains <- 1
+    n_iter_warmup <- 0
+    n_iter_sample <- niterations(csfit$draws)
+    if (n_iter_sample > 0) {
+      csfit$draws <- as_draws_df(csfit$draws)
+    }
+
+    # called 'samples' for consistency with rstan
+    samples <- csfit$draws
+    # manage memory
+    csfit$draws <- NULL
+
+    # VI has no sampler diagnostics
+    diag_chain_ids <- rep(1L, nrow(samples))
+    diagnostics <- as.data.frame(matrix(nrow = nrow(samples), ncol = 0))
+  }
+
+  # convert to regular data.frame
+  samples <- as.data.frame(samples)
+  chain_ids <- samples$.chain
+  samples[res_vars] <- NULL
+
+  # only add special variables to dims if there are present in samples
+  # this ensures that dims_oi, pars_oi, and fnames_oi match with samples
+  for (p in special_vars) {
+    if (p %in% colnames(samples)) {
+      samples <- move2end(samples, p)
+      par_dims[[p]] <- integer(0)
+    }
+  }
+  model_pars <- names(par_dims)
+  fnames_oi <- colnames(samples)
+
+  # split samples into chains
+  samples <- split(samples, chain_ids)
+  names(samples) <- NULL
+
+  # split diagnostics into chains
+  diagnostics <- split(diagnostics, diag_chain_ids)
+  names(diagnostics) <- NULL
+
+  #  @sim$sample: largely 113-130 from rstan::read_stan_csv
+  values <- list()
+  values$algorithm <- csfit$metadata$algorithm
+  values$engine <- csfit$metadata$engine
+  values$metric <- csfit$metadata$metric
+
+  sampler_t <- NULL
+  if (!is.null(values$algorithm)) {
+    if (values$algorithm == "rwm" || values$algorithm == "Metropolis") {
+      sampler_t <- "Metropolis"
+    } else if (values$algorithm == "hmc") {
+      if (values$engine == "static") {
+        sampler_t <- "HMC"
+      } else {
+        if (values$metric == "unit_e") {
+          sampler_t <- "NUTS(unit_e)"
+        } else if (values$metric == "diag_e") {
+          sampler_t <- "NUTS(diag_e)"
+        } else if (values$metric == "dense_e") {
+          sampler_t <- "NUTS(dense_e)"
+        }
+      }
+    }
+  }
+
+  adapt_info <- vector("list", 4)
+  idx_samples <- (n_iter_warmup + 1):(n_iter_warmup + n_iter_sample)
+
+  for (i in seq_along(samples)) {
+    m <- colMeans(samples[[i]][idx_samples, , drop = FALSE])
+    rownames(samples[[i]]) <- seq_rows(samples[[i]])
+    attr(samples[[i]], "sampler_params") <- diagnostics[[i]][rstan_diagn_order]
+    rownames(attr(samples[[i]], "sampler_params")) <- seq_rows(diagnostics[[i]])
+
+    # reformat back to text
+    if (length(csfit$inv_metric)) {
+      if (is_equal(sampler_t, "NUTS(dense_e)")) {
+        mmatrix_txt <- "\n# Elements of inverse mass matrix:\n# "
+        mmat <- paste0(apply(csfit$inv_metric[[i]], 1, paste0, collapse = ", "),
+                       collapse = "\n# ")
+      } else {
+        mmatrix_txt <- "\n# Diagonal elements of inverse mass matrix:\n# "
+        mmat <- paste0(csfit$inv_metric[[i]], collapse = ", ")
+      }
+
+      adapt_info[[i]] <- paste0("# Step size = ",
+                                csfit$step_size[[i]],
+                                mmatrix_txt,
+                                mmat, "\n# ")
+      attr(samples[[i]], "adaptation_info") <- adapt_info[[i]]
+    } else {
+      attr(samples[[i]], "adaptation_info") <- character(0)
+    }
+
+
+    attr(samples[[i]], "args") <- list(sampler_t = sampler_t, chain_id = i)
+
+    if (NROW(csfit$metadata$time)) {
+      time_i <- as.double(csfit$metadata$time[i, c("warmup", "sampling")])
+      names(time_i) <- c("warmup", "sample")
+      attr(samples[[i]], "elapsed_time") <- time_i
+    }
+
+    attr(samples[[i]], "mean_pars") <- m[-length(m)]
+    attr(samples[[i]], "mean_lp__") <- m["lp__"]
+  }
+
+  perm_lst <- lapply(seq_len(n_chains), function(id) sample.int(n_iter_sample))
+
+  # @sim
+  sim <- list(
+    samples = samples,
+    iter = csfit$metadata$iter_sampling + csfit$metadata$iter_warmup,
+    thin = csfit$metadata$thin,
+    warmup = csfit$metadata$iter_warmup,
+    chains = n_chains,
+    n_save = rep(n_iter_sample + n_iter_warmup, n_chains),
+    warmup2 = rep(n_iter_warmup, n_chains),
+    permutation = perm_lst,
+    pars_oi = model_pars,
+    dims_oi = par_dims,
+    fnames_oi = fnames_oi,
+    n_flatnames = length(fnames_oi)
+  )
+
+  # @stan_args
+  sargs <- list(
+    stan_version_major = as.character(csfit$metadata$stan_version_major),
+    stan_version_minor = as.character(csfit$metadata$stan_version_minor),
+    stan_version_patch = as.character(csfit$metadata$stan_version_patch),
+    model = csfit$metadata$model_name,
+    start_datetime = gsub(" ", "", csfit$metadata$start_datetime),
+    method = csfit$metadata$method,
+    iter = csfit$metadata$iter_sampling + csfit$metadata$iter_warmup,
+    warmup = csfit$metadata$iter_warmup,
+    save_warmup = csfit$metadata$save_warmup,
+    thin = csfit$metadata$thin,
+    engaged = as.character(csfit$metadata$adapt_engaged),
+    gamma = csfit$metadata$gamma,
+    delta = csfit$metadata$adapt_delta,
+    kappa = csfit$metadata$kappa,
+    t0 = csfit$metadata$t0,
+    init_buffer = as.character(csfit$metadata$init_buffer),
+    term_buffer = as.character(csfit$metadata$term_buffer),
+    window = as.character(csfit$metadata$window),
+    algorithm = csfit$metadata$algorithm,
+    engine = csfit$metadata$engine,
+    max_depth = csfit$metadata$max_treedepth,
+    metric = csfit$metadata$metric,
+    metric_file = character(0), # not stored in metadata
+    stepsize = NA, # add in loop
+    stepsize_jitter = csfit$metadata$stepsize_jitter,
+    num_chains = as.character(csfit$metadata$num_chains),
+    chain_id = NA, # add in loop
+    file = character(0), # not stored in metadata
+    init = NA, # add in loop
+    seed = as.character(csfit$metadata$seed),
+    file = NA, # add in loop
+    diagnostic_file = character(0), # not stored in metadata
+    refresh = as.character(csfit$metadata$refresh),
+    sig_figs = as.character(csfit$metadata$sig_figs),
+    profile_file = csfit$metadata$profile_file,
+    num_threads = as.character(csfit$metadata$threads_per_chain),
+    stanc_version = gsub(" ", "", csfit$metadata$stanc_version),
+    stancflags = character(0), # not stored in metadata
+    adaptation_info = NA, # add in loop
+    has_time = is.numeric(csfit$metadata$time$total),
+    time_info = NA, # add in loop
+    sampler_t = sampler_t
+  )
+
+  sargs_rep <- replicate(n_chains, sargs, simplify = FALSE)
+
+  for (i in seq_along(sargs_rep)) {
+    sargs_rep[[i]]$chain_id <- i
+    sargs_rep[[i]]$stepsize <- csfit$metadata$step_size[i]
+    sargs_rep[[i]]$init <- as.character(csfit$metadata$init[i])
+    # two 'file' elements: select the second
+    file_idx <- which(names(sargs_rep[[i]]) == "file")
+    sargs_rep[[i]][[file_idx[2]]] <- files[[i]]
+
+    sargs_rep[[i]]$adaptation_info <- adapt_info[[i]]
+
+    if (NROW(csfit$metadata$time)) {
+      sargs_rep[[i]]$time_info <- paste0(
+        c("#  Elapsed Time: ", "#                ", "#                ", "# "),
+        c(csfit$metadata$time[i, c("warmup", "sampling", "total")], ""),
+        c(" seconds (Warm-up)", " seconds (Sampling)", " seconds (Total)", "")
+      )
+    }
+  }
+
+  # @stanmodel
+  null_dso <- new(
+    "cxxdso", sig = list(character(0)), dso_saved = FALSE,
+    dso_filename = character(0), modulename = character(0),
+    system = R.version$system, cxxflags = character(0),
+    .CXXDSOMISC = new.env(parent = emptyenv())
+  )
+  null_sm <- new(
+    "stanmodel", model_name = model_name, model_code = character(0),
+    model_cpp = list(), dso = null_dso
+  )
+
+  # @date
+  sdate <- do.call(max, lapply(files, function(csv) file.info(csv)$mtime))
+  sdate <- format(sdate, "%a %b %d %X %Y")
+
+  out <- new(
+    "stanfit",
+    model_name = model_name,
+    model_pars = model_pars,
+    par_dims = par_dims,
+    mode = mode,
+    sim = sim,
+    inits = list(),
+    stan_args = sargs_rep,
+    stanmodel = null_sm,
+    date = sdate,  # not the time of sampling
+    .MISC = new.env(parent = emptyenv())
+  )
+
+  attributes(out)$metadata <- csfit
+  attributes(out)$CmdStanModel <- model
+  out
 }
