@@ -107,10 +107,11 @@ exclude_pars.btl <- function(x, data, save_pars, ...) {
   p <- usc(combine_prefix(x))
   c(out) <- paste0("chol_cor", p)
   if (!save_pars$all) {
+    # removed the "Intercept" and "first_Intercept" parameters from this list
+    # to reduce the number of models that need refitting for moment matching
     par_classes <- c(
       "bQ", "zb", "zbsp", "zbs", "zar", "zma", "hs_local", "R2D2_phi",
-      "scales", "Intercept", "first_Intercept", "merged_Intercept",
-      "zcar", "nszcar", "zerr"
+      "scales", "merged_Intercept", "zcar", "nszcar", "zerr"
     )
     c(out) <- paste0(par_classes, p)
     smef <- tidy_smef(x, data)
@@ -125,7 +126,7 @@ exclude_pars.btl <- function(x, data, save_pars, ...) {
 #' Control Saving of Parameter Draws
 #'
 #' Control which (draws of) parameters should be saved in a \pkg{brms}
-#' model. The output of this function is ment for usage in the
+#' model. The output of this function is meant for usage in the
 #' \code{save_pars} argument of \code{\link{brm}}.
 #'
 #' @param group A flag to indicate if group-level coefficients for
