@@ -11,6 +11,7 @@
 #   normalized: suffixes of Stan lpdfs or lpmfs which only exist as normalized
 #     versions; can also be "" in which case the family is always normalized
 #   specials: character vector specialties of some families
+#     TODO: create an overview of all specials
 
 .family_gaussian <- function() {
   list(
@@ -50,7 +51,7 @@
 .family_binomial <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", "cloglog", 
+      "logit", "probit", "probit_approx", "cloglog",
       "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu"), type = "int",
@@ -75,7 +76,7 @@
 .family_bernoulli <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", "cloglog", 
+      "logit", "probit", "probit_approx", "cloglog",
       "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu"), type = "int",
@@ -114,7 +115,7 @@
 .family_beta <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", "cloglog", 
+      "logit", "probit", "probit_approx", "cloglog",
       "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu", "phi"), type = "real",
@@ -181,7 +182,14 @@
     dpars = c("mu", "shape"), type = "int",
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "cens", "trunc", "rate", "index"),
-    specials = "sbi_log"
+    specials = "sbi_log",
+    # experimental use of default priors stored in families #1614
+    prior = function(dpar, link = "identity", ...) {
+      if (dpar == "shape" && link == "identity") {
+        return("inv_gamma(0.4, 0.3)")
+      }
+      NULL
+    }
   )
 }
 
@@ -192,7 +200,13 @@
     dpars = c("mu", "sigma"), type = "int",
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "cens", "trunc", "rate", "index"),
-    specials = "sbi_log"
+    specials = "sbi_log",
+    prior = function(dpar, link = "identity", ...) {
+      if (dpar == "sigma" && link == "identity") {
+        return("gamma(0.4, 0.3)")
+      }
+      NULL
+    }
   )
 }
 
@@ -235,8 +249,7 @@
     links = c("log", "identity", "inverse", "softplus", "squareplus"),
     dpars = c("mu", "shape"), type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
-    ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
-    specials = "transeta"  # see stan_eta_inv_link()
+    ad = c("weights", "subset", "cens", "trunc", "mi", "index")
   )
 }
 
@@ -245,8 +258,7 @@
     links = c("log", "identity", "inverse", "softplus", "squareplus"),
     dpars = c("mu", "shape"), type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
-    ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
-    specials = "transeta"  # see stan_eta_inv_link()
+    ad = c("weights", "subset", "cens", "trunc", "mi", "index")
   )
 }
 
@@ -255,8 +267,7 @@
     links = c("log", "identity", "inverse", "softplus", "squareplus"),
     dpars = "mu", type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
-    ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
-    specials = "transeta"  # see stan_eta_inv_link()
+    ad = c("weights", "subset", "cens", "trunc", "mi", "index")
   )
 }
 
@@ -266,8 +277,7 @@
     dpars = c("mu", "nu"), type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
     ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
-    include = "fun_logm1.stan",
-    specials = "transeta"  # see stan_eta_inv_link()
+    include = "fun_logm1.stan"
   )
 }
 
@@ -296,7 +306,7 @@
     links = c("identity", "inverse"),
     dpars = c("mu", "sigma", "ndt"), type = "real",
     ybounds = c(0, Inf), closed = c(FALSE, NA),
-    ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
+    ad = c("weights", "subset", "cens", "trunc", "index"),
     specials = "logscale"
   )
 }
@@ -324,7 +334,7 @@
 .family_gen_extreme_value <- function() {
   list(
     links = c("identity", "log", "inverse", "softplus", "squareplus"),
-    dpars = c("mu", "sigma", "xi"), 
+    dpars = c("mu", "sigma", "xi"),
     tmp_dpars = "xi", type = "real",
     ybounds = c(-Inf, Inf), closed = c(NA, NA),
     ad = c("weights", "subset", "cens", "trunc", "mi", "index"),
@@ -467,7 +477,7 @@
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "cens", "trunc", "index"),
     include = "fun_hurdle_gamma.stan",
-    specials = "sbi_hu_logit",
+    specials = c("sbi_hu_logit", "cont_hurdle"),
     normalized = ""
   )
 }
@@ -479,7 +489,24 @@
     ybounds = c(0, Inf), closed = c(TRUE, NA),
     ad = c("weights", "subset", "cens", "trunc", "index"),
     include = "fun_hurdle_lognormal.stan",
-    specials = c("logscale", "sbi_hu_logit"),
+    specials = c("logscale", "sbi_hu_logit", "cont_hurdle"),
+    normalized = ""
+  )
+}
+
+.family_hurdle_cumulative <- function() {
+  list(
+    links = c(
+      "logit", "probit", "probit_approx",
+      "cloglog", "cauchit", "softit"
+    ),
+    dpars = c("mu", "hu", "disc"), type = "int",
+    ybounds = c(-Inf, Inf), closed = c(NA, NA),
+    ad = c("weights", "subset", "thres", "cat", "index"),
+    specials = c(
+      "ordinal", "ordered_thres", "thres_minus_eta",
+      "joint_link", "ocs", "sbi_logit", "extra_cat"
+    ),
     normalized = ""
   )
 }
@@ -511,7 +538,7 @@
 .family_zero_inflated_binomial <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", "cloglog", 
+      "logit", "probit", "probit_approx", "cloglog",
       "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu", "zi"), type = "int",
@@ -526,7 +553,7 @@
 .family_zero_inflated_beta_binomial <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", "cloglog", 
+      "logit", "probit", "probit_approx", "cloglog",
       "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu", "phi", "zi"),
@@ -543,14 +570,14 @@
 .family_zero_inflated_beta <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", "cloglog", 
+      "logit", "probit", "probit_approx", "cloglog",
       "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu", "phi", "zi"), type = "real",
     ybounds = c(0, 1), closed = c(TRUE, FALSE),
     ad = c("weights", "subset", "cens", "trunc", "index"),
     include = "fun_zero_inflated_beta.stan",
-    specials = "sbi_zi_logit",
+    specials = c("sbi_zi_logit", "cont_hurdle"),
     normalized = ""
   )
 }
@@ -558,14 +585,14 @@
 .family_zero_one_inflated_beta <- function() {
   list(
     links = c(
-      "logit", "probit", "probit_approx", "cloglog", 
+      "logit", "probit", "probit_approx", "cloglog",
       "cauchit", "softit", "identity", "log"
     ),
     dpars = c("mu", "phi", "zoi", "coi"), type = "real",
     ybounds = c(0, 1), closed = c(TRUE, TRUE),
     ad = c("weights", "subset", "index"),
     include = "fun_zero_one_inflated_beta.stan",
-    specials = "sbi_zi_logit",
+    specials = c("sbi_zi_logit", "cont_hurdle"),
     normalized = ""
   )
 }
