@@ -359,7 +359,7 @@ tidy_spef <- function(x, data) {
     return(empty_data_frame())
   }
   mm <- sp_model_matrix(form, data, rename = FALSE)
-  out <- data.frame(term = trim_wsp(colnames(mm)), stringsAsFactors = FALSE)
+  out <- data.frame(term = colnames(mm), stringsAsFactors = FALSE)
   out$coef <- rename(out$term)
   calls_cols <- c(paste0("calls_", all_sp_types()), "joint_call")
   list_cols <- c("vars_mi", "idx_mi", "idx2_mi", "ids_mo", "Imo")
@@ -413,7 +413,6 @@ tidy_spef <- function(x, data) {
     has_sp_calls <- grepl_expr(regex_sp(all_sp_types()), terms_split[[i]])
     sp_calls <- sub("^I\\(", "(", terms_split[[i]][has_sp_calls])
     out$joint_call[[i]] <- paste0(sp_calls, collapse = " * ")
-    out$Ic[i] <- any(!has_sp_calls)
   }
 
   # extract data frame to track all required index variables
@@ -439,8 +438,10 @@ tidy_spef <- function(x, data) {
   }
 
   # extract information on covariates
+  # only non-zero covariates are relevant to consider
   not_one <- apply(mm, 2, function(x) any(x != 1))
-  out$Ic <- cumsum(out$Ic | not_one)
+  cumsum_not_one <- cumsum(not_one)
+  out$Ic <- ifelse(not_one, cumsum_not_one, 0)
   out
 }
 
@@ -517,6 +518,8 @@ sp_model_matrix <- function(formula, data, types = all_sp_types(), ...) {
   new_formula <- str2formula(terms_comb)
   attributes(new_formula) <- attributes(formula)
   out <- get_model_matrix(new_formula, data, ...)
+  # fixes issue #1504
+  colnames(out) <- rm_wsp(colnames(out))
   # recover original column names
   colnames(out) <- rename(colnames(out), dummies, terms_replace)
   out
@@ -577,6 +580,7 @@ get_mo_values <- function(term, data) {
   x <- eval2(term$term, data)
   if (is.ordered(x)) {
     # counting starts at zero
+    max_value <- length(levels(x)) - 1
     x <- as.numeric(x) - 1
   } else if (all(is_wholenumber(x))) {
     min_value <- attr(x, "min")
@@ -584,13 +588,16 @@ get_mo_values <- function(term, data) {
       min_value <- min(x)
     }
     x <- x - min_value
+    max_value <- max(x)
   } else {
     stop2(
       "Monotonic predictors must be integers or ordered ",
       "factors. Error occurred for variable '", term$term, "'."
     )
   }
-  as.array(x)
+  x <- as.array(x)
+  attr(x, "max") <- max_value
+  x
 }
 
 # prepare 'sp_term' objects
