@@ -55,6 +55,7 @@ prepare_predictions.brmsfit <- function(
   )
 }
 
+#' @export
 prepare_predictions.mvbrmsterms <- function(x, draws, sdata, resp = NULL, ...) {
   resp <- validate_resp(resp, x$responses)
   if (length(resp) > 1) {
@@ -103,7 +104,7 @@ prepare_predictions.brmsterms <- function(x, draws, sdata, data, ...) {
   out$family <- prepare_family(x)
 
   out$old_order <- attr(sdata, "old_order")
-  if (has_subset(x)) {
+  if (has_subset(x) && !is.null(out$old_order)) {
     # old_order has length equal to the full number of observations
     # which is inappropriate for subsetted responses (#1483)
     out$old_order <- as.numeric(factor(out$old_order[attr(data, "subset")]))
@@ -196,7 +197,9 @@ prepare_predictions.brmsterms <- function(x, draws, sdata, data, ...) {
 #' @export
 prepare_predictions.btnl <- function(x, draws, sdata, ...) {
   out <- list(
-    family = x$family, nlform = x$formula[[2]],
+    family = x$family,
+    nlform = x$formula[[2]],
+    env = environment(x$formula),
     ndraws = nrow(draws),
     nobs = sdata[[paste0("N", usc(x$resp))]],
     used_nlpars = x$used_nlpars,
@@ -245,7 +248,7 @@ prepare_predictions_fe <- function(bterms, draws, sdata, ...) {
   if (length(fixef)) {
     out$X <- X
     b_pars <- paste0("b", p, "_", fixef)
-    out$b <- prepare_draws(draws, b_pars)
+    out$b <- prepare_draws(draws, b_pars, scalar = TRUE)
   }
   out
 }
@@ -1198,7 +1201,9 @@ is.bprepnl <- function(x) {
 #'   out-of-sample rather than in-sample predictions. Only required in models
 #'   that make use of response values to make predictions, that is, currently
 #'   only ARMA models.
-#' @param smooths_only Logical; If \code{TRUE} only predictions related to the
+#' @param smooths_only Logical; If \code{TRUE} only predictions related to
+#'   smoothing splines (i.e., \code{s} or \code{t2}) will be computed.
+#'   Defaults to \code{FALSE}.
 #' @param resp Optional names of response variables. If specified, predictions
 #'   are performed only for the specified response variables.
 #' @param ndraws Positive integer indicating how many posterior draws should
