@@ -24,11 +24,8 @@ stan_global_defs <- function(bterms, prior, ranef, threads) {
   } else if (any(links == "softit")) {
     str_add(out$fun) <- "  #include 'fun_softit.stan'\n"
   }
-  special <- get_special_prior(prior)
-  if (!isNULL(lapply(special, "[[", "horseshoe"))) {
+  if (has_special_prior(prior)) {
     str_add(out$fun) <- "  #include 'fun_horseshoe.stan'\n"
-  }
-  if (!isNULL(lapply(special, "[[", "R2D2"))) {
     str_add(out$fun) <- "  #include 'fun_r2d2.stan'\n"
   }
   if (nrow(ranef)) {
@@ -151,117 +148,65 @@ stan_global_defs <- function(bterms, prior, ranef, threads) {
 
 # link function in Stan language
 # @param link name of the link function
-# @param vectorize use vectorize version of the link function?
 # @param transform actually apply the link function?
-stan_link <- function(link, vectorize = TRUE, transform = TRUE) {
-  vectorize <- as_one_logical(vectorize)
+stan_link <- function(link, transform = TRUE) {
   transform <- as_one_logical(transform %||% FALSE)
   if (!transform) {
     # we have a Stan lpdf that applies the link automatically
     # or we have a non-linear parameter that has no link function
     return("")
   }
-  if (vectorize) {
-    # custom function cannot yet be overloaded in old Stan versions
-    # TODO: change names once overloading is possible in rstan
-    out <- switch(
-      link,
-      identity = "",
-      log = "log",
-      logm1 = "logm1_vector",
-      inverse = "inv",
-      sqrt = "sqrt",
-      "1/mu^2" = "inv_square",
-      logit = "logit",
-      probit = "inv_Phi",
-      probit_approx = "inv_Phi",
-      cloglog = "cloglog_vector",
-      cauchit = "cauchit_vector",
-      tan_half = "tan_half_vector",
-      log1p = "log1p",
-      softplus = "log_expm1_vector",
-      squareplus = "inv_squareplus_vector",
-      softit = "softit_vector"
-    )
-  } else {
-    out <- switch(
-      link,
-      identity = "",
-      log = "log",
-      logm1 = "logm1",
-      inverse = "inv",
-      sqrt = "sqrt",
-      "1/mu^2" = "inv_square",
-      logit = "logit",
-      probit = "inv_Phi",
-      probit_approx = "inv_Phi",
-      cloglog = "cloglog",
-      cauchit = "cauchit",
-      tan_half = "tan_half",
-      log1p = "log1p",
-      softplus = "log_expm1",
-      squareplus = "inv_squareplus",
-      softit = "softit"
-    )
-  }
+  out <- switch(
+    link,
+    identity = "",
+    log = "log",
+    logm1 = "logm1",
+    inverse = "inv",
+    sqrt = "sqrt",
+    "1/mu^2" = "inv_square",
+    logit = "logit",
+    probit = "inv_Phi",
+    probit_approx = "inv_Phi",
+    cloglog = "cloglog",
+    cauchit = "cauchit",
+    tan_half = "tan_half",
+    log1p = "log1p",
+    softplus = "log_expm1",
+    squareplus = "inv_squareplus",
+    softit = "softit"
+  )
   out
 }
 
 # inverse link in Stan language
 # @param link name of the link function
-# @param vectorize use vectorize version of the inv_link function?
 # @param transform actually apply the inv_link function?
-stan_inv_link <- function(link, vectorize = TRUE, transform = TRUE) {
-  vectorize <- as_one_logical(vectorize)
+stan_inv_link <- function(link, transform = TRUE) {
   transform <- as_one_logical(transform %||% FALSE)
   if (!transform) {
     # we have a Stan lpdf that applies the inv_link automatically
     # or we have a non-linear parameter that has no link function
     return("")
   }
-  if (vectorize) {
-    # custom function cannot yet be overloaded in old Stan versions
-    # TODO: change names once overloading is possible in rstan
-    out <- switch(
-      link,
-      identity = "",
-      log = "exp",
-      logm1 = "expp1_vector",
-      inverse = "inv",
-      sqrt = "square",
-      "1/mu^2" = "inv_sqrt",
-      logit = "inv_logit",
-      probit = "Phi",
-      probit_approx = "Phi_approx",
-      cloglog = "inv_cloglog",
-      cauchit = "inv_cauchit_vector",
-      tan_half = "inv_tan_half_vector",
-      log1p = "expm1",
-      softplus = "log1p_exp",
-      squareplus = "squareplus_vector",
-      softit = "inv_softit_vector"
-    )
-  } else {
-    out <- switch(
-      link,
-      identity = "",
-      log = "exp",
-      logm1 = "expp1",
-      inverse = "inv",
-      sqrt = "square",
-      "1/mu^2" = "inv_sqrt",
-      logit = "inv_logit",
-      probit = "Phi",
-      probit_approx = "Phi_approx",
-      cloglog = "inv_cloglog",
-      cauchit = "inv_cauchit",
-      tan_half = "inv_tan_half",
-      log1p = "expm1",
-      softplus = "log1p_exp",
-      squareplus = "squareplus",
-      softit = "inv_softit"
-    )
-  }
+  out <- switch(
+    link,
+    identity = "",
+    log = "exp",
+    logm1 = "expp1",
+    inverse = "inv",
+    sqrt = "square",
+    "1/mu^2" = "inv_sqrt",
+    logit = "inv_logit",
+    probit = "Phi",
+    probit_approx = "Phi_approx",
+    cloglog = "inv_cloglog",
+    cauchit = "inv_cauchit",
+    tan_half = "inv_tan_half",
+    log1p = "expm1",
+    softplus = "log1p_exp",
+    squareplus = "squareplus",
+    softit = "inv_softit"
+  )
   out
 }
 
