@@ -68,7 +68,7 @@ stan_log_lik.mixfamily <- function(x, bterms, threads, ...) {
     "  // likelihood of the mixture model\n",
     "  for (n in 1:N{resp}) {{\n",
     stan_nn_def(threads),
-    "    real ps[{length(ll)}];\n"
+    "    array[{length(ll)}] real ps;\n"
   )
   str_add(out) <- collapse("    ", ll)
   str_add(out) <- glue(
@@ -310,7 +310,8 @@ stan_log_lik_add_se <- function(sigma, bterms, reqn, resp = "",
 # multiply 'dpar' by the 'rate' denominator within the Stan likelihood
 # @param log add the rate denominator on the log scale if sensible?
 stan_log_lik_multiply_rate_denom <- function(dpar, bterms, reqn, resp = "",
-                                             log = FALSE, transform = NULL) {
+                                             log = FALSE, transform = NULL,
+                                             threads = NULL) {
   dpar_transform <- dpar
   if (!is.null(transform)) {
     dpar_transform <- glue("{transform}({dpar})")
@@ -318,7 +319,7 @@ stan_log_lik_multiply_rate_denom <- function(dpar, bterms, reqn, resp = "",
   if (!is.formula(bterms$adforms$rate)) {
     return(dpar_transform)
   }
-  ndenom <- str_if(reqn, "[n]")
+  ndenom <- str_if(reqn, stan_nn(threads), stan_slice(threads))
   denom <- glue("denom{resp}{ndenom}")
   cens_or_trunc <- stan_log_lik_adj(bterms, c("cens", "trunc"))
   if (log && bterms$family$link == "log" && !cens_or_trunc) {
@@ -528,7 +529,9 @@ stan_log_lik_poisson <- function(bterms, resp = "", mix = "", threads = NULL,
     reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
     p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
     lpdf <- stan_log_lik_simple_lpdf("poisson", "log", bterms)
-    p$mu <- stan_log_lik_multiply_rate_denom(p$mu, bterms, reqn, resp, log = TRUE)
+    p$mu <- stan_log_lik_multiply_rate_denom(
+      p$mu, bterms, reqn, resp, log = TRUE, threads = threads
+    )
     out <- sdist(lpdf, p$mu)
   }
   out
@@ -543,8 +546,12 @@ stan_log_lik_negbinomial <- function(bterms, resp = "", mix = "", threads = NULL
   } else {
     reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
     p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
-    p$mu <- stan_log_lik_multiply_rate_denom(p$mu, bterms, reqn, resp, log = TRUE)
-    p$shape <- stan_log_lik_multiply_rate_denom(p$shape, bterms, reqn, resp)
+    p$mu <- stan_log_lik_multiply_rate_denom(
+      p$mu, bterms, reqn, resp, log = TRUE, threads = threads
+    )
+    p$shape <- stan_log_lik_multiply_rate_denom(
+      p$shape, bterms, reqn, resp, threads = threads
+    )
     lpdf <- stan_log_lik_simple_lpdf("neg_binomial_2", "log", bterms)
     out <- sdist(lpdf, p$mu, p$shape)
   }
@@ -561,9 +568,11 @@ stan_log_lik_negbinomial2 <- function(bterms, resp = "", mix = "", threads = NUL
   } else {
     reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
     p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
-    p$mu <- stan_log_lik_multiply_rate_denom(p$mu, bterms, reqn, resp, log = TRUE)
+    p$mu <- stan_log_lik_multiply_rate_denom(
+      p$mu, bterms, reqn, resp, log = TRUE, threads = threads
+    )
     p$shape <- stan_log_lik_multiply_rate_denom(
-      p$sigma, bterms, reqn, resp, transform = "inv"
+      p$sigma, bterms, reqn, resp, transform = "inv", threads = threads
     )
     lpdf <- stan_log_lik_simple_lpdf("neg_binomial_2", "log", bterms)
     out <- sdist(lpdf, p$mu, p$shape)
@@ -581,8 +590,12 @@ stan_log_lik_geometric <- function(bterms, resp = "", mix = "", threads = NULL,
     reqn <- stan_log_lik_adj(bterms) || nzchar(mix)
     p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
     p$shape <- "1"
-    p$mu <- stan_log_lik_multiply_rate_denom(p$mu, bterms, reqn, resp, log = TRUE)
-    p$shape <- stan_log_lik_multiply_rate_denom(p$shape, bterms, reqn, resp)
+    p$mu <- stan_log_lik_multiply_rate_denom(
+      p$mu, bterms, reqn, resp, log = TRUE, threads = threads
+    )
+    p$shape <- stan_log_lik_multiply_rate_denom(
+      p$shape, bterms, reqn, resp, threads = threads
+    )
     lpdf <- stan_log_lik_simple_lpdf("neg_binomial_2", "log", bterms)
     out <- sdist(lpdf, p$mu, p$shape)
   }
@@ -689,9 +702,8 @@ stan_log_lik_inverse.gaussian <- function(bterms, resp = "", mix = "", ...) {
   reqn <- stan_log_lik_adj(bterms) || nzchar(mix) ||
     glue("shape{mix}") %in% names(bterms$dpars)
   p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
-  lpdf <- paste0("inv_gaussian", if (!reqn) "_vector")
   n <- str_if(reqn, "[n]")
-  sdist(lpdf, p$mu, p$shape)
+  sdist("inv_gaussian", p$mu, p$shape)
 }
 
 stan_log_lik_wiener <- function(bterms, resp = "", mix = "", threads = NULL,
@@ -716,8 +728,7 @@ stan_log_lik_von_mises <- function(bterms, resp = "", mix = "", ...) {
   reqn <- stan_log_lik_adj(bterms) || nzchar(mix) ||
     "kappa" %in% names(bterms$dpars)
   p <- stan_log_lik_dpars(bterms, reqn, resp, mix)
-  lpdf <- paste0("von_mises_", str_if(reqn, "real", "vector"))
-  sdist(lpdf, p$mu, p$kappa)
+  sdist("von_mises2", p$mu, p$kappa)
 }
 
 stan_log_lik_cox <- function(bterms, resp = "", mix = "", threads = NULL,
@@ -734,12 +745,13 @@ stan_log_lik_cox <- function(bterms, resp = "", mix = "", threads = NULL,
 
 stan_log_lik_cumulative <- function(bterms, resp = "", mix = "",
                                     threads = NULL, ...) {
-  if (use_glm_primitive(bterms, allow_special_terms = FALSE)) {
+  if (use_glm_primitive(bterms)) {
     p <- args_glm_primitive(bterms$dpars$mu, resp = resp, threads = threads)
     out <- sdist("ordered_logistic_glm", p$x, p$beta, p$alpha)
-    return(out)
+  } else {
+    out <- stan_log_lik_ordinal(bterms, resp, mix, threads, ...)
   }
-  stan_log_lik_ordinal(bterms, resp, mix, threads, ...)
+  out
 }
 
 stan_log_lik_sratio <- function(bterms, resp = "", mix = "",
@@ -757,11 +769,20 @@ stan_log_lik_acat <- function(bterms, resp = "", mix = "",
   stan_log_lik_ordinal(bterms, resp, mix, threads, ...)
 }
 
-stan_log_lik_categorical <- function(bterms, resp = "", mix = "", ...) {
+stan_log_lik_categorical <- function(bterms, resp = "", mix = "",
+                                     threads = NULL, ...) {
   stopifnot(bterms$family$link == "logit")
   stopifnot(!isTRUE(nzchar(mix)))  # mixture models are not allowed
-  p <- stan_log_lik_dpars(bterms, TRUE, resp, mix, dpars = "mu", type = "multi")
-  sdist("categorical_logit", p$mu)
+  if (use_glm_primitive_categorical(bterms)) {
+    bterms1 <- bterms$dpars[[1]]
+    bterms1$family <- bterms$family
+    p <- args_glm_primitive(bterms1, resp = resp, threads = threads)
+    out <- sdist("categorical_logit_glm", p$x, p$alpha, p$beta)
+  } else {
+    p <- stan_log_lik_dpars(bterms, TRUE, resp, mix, dpars = "mu", type = "multi")
+    out <- sdist("categorical_logit", p$mu)
+  }
+  out
 }
 
 stan_log_lik_multinomial <- function(bterms, resp = "", mix = "", ...) {
@@ -799,10 +820,12 @@ stan_log_lik_ordinal <- function(bterms, resp = "", mix = "",
                                  threads = NULL, ...) {
   prefix <- paste0(str_if(nzchar(mix), paste0("_mu", mix)), resp)
   p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
-  if (use_ordered_logistic(bterms)) {
-    # TODO: support 'ordered_probit' as well
+  if (use_ordered_builtin(bterms, "logit")) {
     lpdf <- "ordered_logistic"
     p[grepl("^disc", names(p))] <- NULL
+  } else if (use_ordered_builtin(bterms, "probit")) {
+    lpdf <- "ordered_probit"
+    p[grepl("^disc", names(p))] <- NULL
   } else {
     lpdf <- paste0(bterms$family$family, "_", bterms$family$link)
   }
@@ -861,9 +884,10 @@ stan_log_lik_hurdle_cumulative <- function(bterms, resp = "", mix = "",
                                            threads = NULL, ...) {
   prefix <- paste0(str_if(nzchar(mix), paste0("_mu", mix)), resp)
   p <- stan_log_lik_dpars(bterms, TRUE, resp, mix)
-  if (use_ordered_logistic(bterms)) {
-    # TODO: support 'ordered_probit' as well
+  if (use_ordered_builtin(bterms, "logit")) {
     lpdf <- "hurdle_cumulative_ordered_logistic"
+  } else if (use_ordered_builtin(bterms, "probit")) {
+    lpdf <- "hurdle_cumulative_ordered_probit"
   } else {
     lpdf <- paste0(bterms$family$family, "_", bterms$family$link)
   }
@@ -982,34 +1006,55 @@ stan_log_lik_custom <- function(bterms, resp = "", mix = "", threads = NULL, ...
 # use Stan GLM primitive functions?
 # @param bterms a brmsterms object
 # @return TRUE or FALSE
-use_glm_primitive <- function(bterms, allow_special_terms = TRUE) {
+use_glm_primitive <- function(bterms) {
   stopifnot(is.brmsterms(bterms))
   # the model can only have a single predicted parameter
   # and no additional residual or autocorrelation structure
-  non_glm_adterms <- c("se", "weights", "thres", "cens", "trunc", "rate")
   mu <- bterms$dpars[["mu"]]
+  non_glm_adterms <- c("se", "weights", "thres", "cens", "trunc", "rate")
   if (!is.btl(mu) || length(bterms$dpars) > 1L ||
       isTRUE(bterms$rescor) || is.formula(mu$ac) ||
       any(names(bterms$adforms) %in% non_glm_adterms)) {
     return(FALSE)
   }
+  # some primitives do not support special terms in the way
+  # required by brms' Stan code generation
+  allow_special_terms <- !mu$family$family %in% c("cumulative", "categorical")
+  if (!allow_special_terms && has_special_terms(mu)) {
+    return(FALSE)
+  }
   # supported families and link functions
-  # TODO: support categorical_logit primitive
   glm_links <- list(
     gaussian = "identity", bernoulli = "logit",
-    poisson = "log", negbinomial = "log", negbinomial2 = "log"
-    # rstan does not yet support 'ordered_logistic_glm'
-    # cumulative = "logit"
+    poisson = "log", negbinomial = "log", negbinomial2 = "log",
+    cumulative = "logit", categorical = "logit"
   )
   if (!isTRUE(glm_links[[mu$family$family]] == mu$family$link)) {
     return(FALSE)
   }
-  if (!allow_special_terms && has_special_terms(mu)) {
-    # some primitives do not support special terms in the way
-    # required by brms' Stan code generation
+  length(all_terms(mu$fe)) > 0 && !is_sparse(mu$fe)
+}
+
+# use Stan categorical GLM primitive function?
+# @param bterms a brmsterms object
+# @return TRUE or FALSE
+use_glm_primitive_categorical <- function(bterms) {
+  stopifnot(is.brmsterms(bterms))
+  if (!is_categorical(bterms)) {
     return(FALSE)
   }
-  length(all_terms(mu$fe)) > 0 && !is_sparse(mu$fe)
+  tmp <- bterms
+  tmp$dpars <- list()
+  # we know that all dpars in categorical models are mu parameters
+  out <- rep(FALSE, length(bterms$dpars))
+  for (i in seq_along(bterms$dpars)) {
+    tmp$dpars$mu <- bterms$dpars[[i]]
+    tmp$dpars$mu$family <- bterms$family
+    out[i] <- use_glm_primitive(tmp) &&
+      # the design matrix of all mu parameters must match
+      all.equal(tmp$dpars$mu$fe, bterms$dpars[[1]]$fe)
+  }
+  all(out)
 }
 
 # standard arguments for primitive Stan GLM functions
@@ -1027,6 +1072,10 @@ args_glm_primitive <- function(bterms, resp = "", threads = NULL) {
   } else if (center_X) {
     sfx_X <- "c"
   }
+  is_categorical <- is_categorical(bterms)
+  if (is_categorical) {
+    sfx_X <- glue("{sfx_X}_{bterms$dpar}")
+  }
   x <- glue("X{sfx_X}{resp}{slice}")
   beta <- glue("b{sfx_b}{resp}")
   if (has_special_terms(bterms)) {
@@ -1036,17 +1085,21 @@ args_glm_primitive <- function(bterms, resp = "", threads = NULL) {
     if (center_X) {
       alpha <- glue("Intercept{resp}")
     } else {
-      alpha <- "0"
+      if (is_categorical) {
+        alpha <- glue("rep_vector(0, ncat{resp})")
+      } else {
+        alpha <- "0"
+      }
     }
   }
   nlist(x, alpha, beta)
 }
 
 # use the ordered_logistic built-in functions
-use_ordered_logistic <- function(bterms) {
+use_ordered_builtin <- function(bterms, link) {
   stopifnot(is.brmsterms(bterms))
   isTRUE(bterms$family$family %in% c("cumulative", "hurdle_cumulative")) &&
-    isTRUE(bterms$family$link == "logit") &&
+    isTRUE(bterms$family$link == link) &&
     isTRUE(bterms$fdpars$disc$value == 1) &&
     !has_cs(bterms)
 }
