@@ -193,23 +193,28 @@ update.brmsfit <- function(object, formula., newdata = NULL,
   }
 
   # update arguments controlling the sampling process
-  if (is.null(dots$iter)) {
-    # only keep old 'warmup' if also keeping old 'iter'
-    dots$warmup <- first_not_null(dots$warmup, object$fit@sim$warmup)
-  }
-  dots$iter <- first_not_null(dots$iter, object$fit@sim$iter)
-  dots$chains <- first_not_null(dots$chains, object$fit@sim$chains)
-  dots$thin <- first_not_null(dots$thin, object$fit@sim$thin)
+  dots$algorithm <- match.arg(dots$algorithm, algorithm_choices())
   dots$backend <- match.arg(dots$backend, backend_choices())
+  same_algorithm <- is_equal(dots$algorithm, object$algorithm)
   same_backend <- is_equal(dots$backend, object$backend)
-  if (same_backend) {
-    # reusing control arguments in other backends may cause errors #1259
-    control <- attr(object$fit@sim$samples[[1]], "args")$control
-    control <- control[setdiff(names(control), names(dots$control))]
-    dots$control[names(control)] <- control
-    # reuse backend arguments originally passed to brm #1373
-    names_old_stan_args <- setdiff(names(object$stan_args), names(dots))
-    dots[names_old_stan_args] <- object$stan_args[names_old_stan_args]
+  if (same_algorithm) {
+    # reusing sampling arguments in other algorithms may cause errors #1564
+    if (is.null(dots$iter)) {
+      # only keep old 'warmup' if also keeping old 'iter'
+      dots$warmup <- first_not_null(dots$warmup, object$fit@sim$warmup)
+    }
+    dots$iter <- first_not_null(dots$iter, object$fit@sim$iter)
+    dots$chains <- first_not_null(dots$chains, object$fit@sim$chains)
+    dots$thin <- first_not_null(dots$thin, object$fit@sim$thin)
+    if (same_backend) {
+      # reusing control arguments in other backends may cause errors #1259
+      control <- attr(object$fit@sim$samples[[1]], "args")$control
+      control <- control[setdiff(names(control), names(dots$control))]
+      dots$control[names(control)] <- control
+      # reuse backend arguments originally passed to brm #1373
+      names_old_stan_args <- setdiff(names(object$stan_args), names(dots))
+      dots[names_old_stan_args] <- object$stan_args[names_old_stan_args]
+    }
   }
 
   if (is.null(recompile)) {
